# Copyright (C) 2018-2023 Mark McIntyre
#
# simple script to get the active shower list from the IMO working list

import numpy as np
import sys
import os
import datetime

from ukmon_meteortools.utils import jd2Date, sollon2jd
try:
    from UpdateOrbitFiles import updateOrbitFiles ## to update WMPL raw data files
    gotupdater = True
except:
    gotupdater = False


def loadFullData(pth=None):
    return loadDataFile(1, pth)


def loadLookupTable(pth=None):
    return loadDataFile(2, pth)


def loadJenniskensShowers(dir_path, file_name):
    """ Load the showers from the Jenniskens et al. (2018) table and init MeteorShower objects. """
    jenniskens_shower_list = []
    with open(os.path.join(dir_path, file_name), encoding='cp1252') as f:
        data_start = 0
        for line in f:
            if "====================================" in line:
                data_start += 1
                continue
            if data_start < 2:
                continue
            line = line.replace('\n', '').replace('\r', '')
            if not line:
                continue
            if "[FINAL]" in line:
                break
            l0, L_l0, B_g, v_g, IAU_no = line.split()
            jenniskens_shower_list.append([np.radians(float(l0)), np.radians(float(L_l0)), 
                np.radians(float(B_g)), 1000*float(v_g), int(IAU_no)])
    return np.array(jenniskens_shower_list)


def refreshShowerData():
    if gotupdater is True:
        updateOrbitFiles()
    abs_path = os.getenv('WMPL_LOC', default='/home/ec2-user/src/WesternMeteorPyLib')
    jenniskens_shower_table_file = os.path.join(abs_path, 'wmpl', 'share', 'ShowerLookUpTable.txt')
    jenniskens_shower_table_npy = os.path.join(abs_path, 'wmpl', 'share', 'ShowerLookUpTable.npy')
    jenniskens_shower_list = loadJenniskensShowers(*os.path.split(jenniskens_shower_table_file))
    np.save(jenniskens_shower_table_npy, jenniskens_shower_list)
    iau_shower_table_file = os.path.join(abs_path, 'wmpl', 'share', 'streamfulldata.csv')
    iau_shower_table_npy = os.path.join(abs_path, 'wmpl', 'share', 'streamfulldata.npy')
    iau_shower_list = np.loadtxt(iau_shower_table_file, delimiter="|", usecols=range(20), dtype=str)
    np.save(iau_shower_table_npy, iau_shower_list)


def loadDataFile(typ, pth=None):
    if typ == 1:
        fname='streamfulldata.npy'
    elif typ == 2:
        fname='ShowerLookupTable.npy'
    else:
        return 'invalid type code'

    if pth is None:
        if sys.platform == 'win32':
            pth = 'e:/dev/meteorhunting/WesternMeteorPyLib/wmpl/share'
        else:
            pth = '/home/ec2-user/src/WesternMeteorPyLib/wmpl/share'
    dfil = np.load(os.path.join(pth, fname))
    return dfil


def getShowerDets(shwr):
    sfd = loadFullData()
    sfdfltr = sfd[sfd[:,3] == shwr]
    mtch = [sh for sh in sfdfltr if sh[6] != '-2']
    if len(mtch) == 0:
        return 0, 'Unknown', 0, 'Unknown'

    id = int(mtch[-1][1])
    nam = mtch[-1][4].strip()
    pksollong = float(mtch[-1][7])
    dt = datetime.datetime.now()
    yr = dt.year
    mth = dt.month
    jd = sollon2jd(yr, mth, pksollong)
    pkdt = jd2Date(jd, dt_obj=True)
    return id, nam, pksollong, pkdt.strftime('%m-%d')


def getShowerPeak(shwr):
    _, _, _, pk = getShowerDets(shwr)
    return pk

 
if __name__ == '__main__':
    if sys.argv[1] == 'refresh':
        refreshShowerData()
        exit(0)
    else:
        id, nam, sl, dt = getShowerDets(sys.argv[1])
        print('{},{},{},{}'.format(sl, dt, nam, sys.argv[1]))



#
# python module to read the IMO Working Shower short List
#
# Copyright (C) 2018-2023 Mark McIntyre

import xmltodict
import datetime
import os
import numpy as np
from ukmon_meteortools.utils import jd2Date, sollon2jd

# imported from $SRC/share
try:
    from majorminor import majorlist, minorlist
except:
    majorlist = ['QUA', 'LYR', 'ETA', 'CAP', 'SDA', 'PER', 'AUR', 'ORI', 'NTA', 'STA', 'LEO', 'GEM', 'URS']
    minorlist = ['SPE','OCT','DRA','EGE','MON','HYD','COM','NOO']


class IMOshowerList:
    """
    Class that loads and parses the IMO Working Shower list XML file

    Notes:
        During initialisation the class attempt to load IMO_Working_Meteor_Shower_list.xml from
        $DATADIR/share, if it exists and if DATADIR is defined in your environment. Failing this,
        it will use a reference copy that is distributed with this library. If you want to force
        the library to use a newer version, define DATADIR and put the file in $DATADIR/share.

        Not all known showers are in the IMO working list. If a shower is not in the Working List then 
        this library will reference the full shower list curated by Peter Jenniskens. 
        This contains debated and unconfirmed showers. 

    """
    def __init__(self, fname=None, fullstreamname=None):
        if fname is None:
            datadir = os.getenv('DATADIR', default='/home/ec2-user/prod/data')
            fname = os.path.join(datadir, '..', 'share', 'IMO_Working_Meteor_Shower_List.xml')
            if not os.path.isfile(fname):
                datadir=os.path.split(os.path.abspath(__file__))[0]
                fname = os.path.join(datadir, '..', 'share', 'IMO_Working_Meteor_Shower_List.xml')
        
        tmplist = xmltodict.parse(open(fname, 'rb').read())
        self.showerlist = tmplist['meteor_shower_list']['shower']
        self.fullstreamdata = None

    
    def getShowerByCode(self, iaucode):
        ds = {'@id':None, 'IAU_code':None,'start':None, 'end':None, 
            'peak':None, 'r':None, 'name':None, 'V':None, 'ZHR':None, 'RA':None, 'DE':None}
        got = None
        for shower in self.showerlist:
            if shower['IAU_code'] == iaucode:
                return shower
        if got is None:
            # load stream full dataset as backup
            if self.fullstreamdata is None:
                wmpldir = os.getenv('WMPL_LOC', default='/home/ec2-user/src/WesternMeteorPyLib')
                fullstreamname = os.path.join(wmpldir, 'wmpl', 'share', 'streamfulldata.npy')
                if not os.path.isfile(fullstreamname):
                    datadir=os.path.split(os.path.abspath(__file__))[0]
                    fullstreamname = os.path.join(datadir, '..', 'share', 'streamfulldata.npy')
                self.fullstreamdata = np.load(fullstreamname)
            subset = self.fullstreamdata[np.where(self.fullstreamdata[:,3]==iaucode)]
            
            if subset is not None:
                mtch = [sh for sh in subset if sh[6] != '-2']
                if len(mtch) > 0:
                    ds['IAU_code'] = mtch[-1][3].strip()
                    ds['name'] = mtch[-1][4].strip()
                    ds['V'] = mtch[-1][12]
                    ds['@id'] = mtch[-1][1]
                    ds['RA'] = mtch[-1][8]
                    ds['DE'] = mtch[-1][9]

                    pksollong = float(mtch[-1][7])
                    dt = datetime.datetime.now()
                    yr = dt.year
                    mth = dt.month
                    jd = sollon2jd(yr, mth, pksollong)
                    pkdt = jd2Date(jd, dt_obj=True)
                    ds['peak'] = pkdt.strftime('%h %d')
                    # start/end pop idx, ZHR not available in the IAU data
                    ds['start'] = (pkdt + datetime.timedelta(days=-2)).strftime('%h %d')
                    ds['end'] = (pkdt + datetime.timedelta(days=2)).strftime('%h %d')
                    ds['r'] = '2.0'
                    ds['ZHR'] = '3.0' 
        return ds

    def getStart(self, iaucode, currdt=None):
        shower = self.getShowerByCode(iaucode)
        if currdt is None:
            now = datetime.datetime.today().year
            mth = datetime.datetime.today().month
        else:
            now = datetime.datetime.strptime(str(currdt), '%Y%m%d')
            mth = now.month
            now = now.year
        startdate = datetime.datetime.strptime(shower['start'], '%b %d')
        if iaucode == 'QUA' and mth !=12:
            # quadrantids straddle yearend
            now = now - 1
        startdate = startdate.replace(year=now)
        return startdate

    def getEnd(self, iaucode, currdt=None):
        shower = self.getShowerByCode(iaucode)
        if currdt is None:
            now = datetime.datetime.today().year
            mth = datetime.datetime.today().month
        else:
            now = datetime.datetime.strptime(str(currdt), '%Y%m%d')
            mth = now.month
            now = now.year
        enddate = datetime.datetime.strptime(shower['end'], '%b %d')
        if iaucode == 'QUA' and mth == 12:
            # quadrantids straddle yearend
            now = now + 1
        enddate = enddate.replace(year=now)
        return enddate

    def getPeak(self, iaucode, currdt=None):
        shower = self.getShowerByCode(iaucode)
        if currdt is None:
            now = datetime.datetime.today().year
            mth = datetime.datetime.today().month
        else:
            now = datetime.datetime.strptime(str(currdt), '%Y%m%d')
            mth = now.month
            now = now.year
        enddate = datetime.datetime.strptime(shower['peak'], '%b %d')
        if iaucode == 'QUA' and mth == 12:
            # quadrantids straddle yearend
            now = now + 1
        enddate = enddate.replace(year=now)
        return enddate

    def getRvalue(self, iaucode):
        shower = self.getShowerByCode(iaucode)
        return shower['r']

    def getName(self, iaucode):
        shower = self.getShowerByCode(iaucode)
        return shower['name']

    def getVelocity(self, iaucode):
        shower = self.getShowerByCode(iaucode)
        return shower['V']

    def getZHR(self, iaucode):
        shower = self.getShowerByCode(iaucode)
        zhr = shower['ZHR']
        if zhr is None:
            return -1
        else:
            return int(zhr)

    def getRaDec(self, iaucode):
        shower = self.getShowerByCode(iaucode)
        return float(shower['RA']), float(shower['DE'])

    def getActiveShowers(self, datetotest, majorOnly=False, inclMinor=False):
        activelist = []
        for shower in self.showerlist:
            shwname = shower['IAU_code']
            start = self.getStart(shwname, datetotest.strftime('%Y%m%d'))
            end = self.getEnd(shwname, datetotest.strftime('%Y%m%d')) + datetime.timedelta(days=3)
            if datetotest > start and datetotest < end:
                if majorOnly is False or (majorOnly is True and shwname in majorlist):
                    activelist.append(shwname)
                elif inclMinor is True and shwname in minorlist:
                    activelist.append(shwname)
        return activelist

    def getMajorShowers(self, includeSpo=False, stringFmt=False):
        majlist = majorlist 
        if includeSpo is True:
            majlist.append('spo')
        if stringFmt is True:
            tmplist = ''
            for shwr in majlist:
                tmplist = tmplist + shwr + ' '
            majlist = tmplist
        return majlist



# Copyright (C) 2018-2023 Mark McIntyre
#
# python script to get all live JPGs for a specified time
#
import os
import sys
import boto3
import datetime
from ukmon_meteortools.fileformats import ReadUFOCapXML as ufoc


def getLiveJpgs(dtstr, outdir=None, create_txt=False, buck_name=None):
    """
    Retrieve live images from the ukmon website that match a pattern

    Arguments:
        dtstr:      [str] Date in YYYYMMDD_HHMMSS format. Partial strings allowed 
        outdir:     [str] Where to save the file. Default is to create a folder named dtstr
        create_txt: [bool] If true, create a text file containing the pattern matches
        buck_name:  [str] S3 bucket to read. Default ukmon-live. 

    Notes:
        This function will fail if you do not have access to the bucket. 
    """
    if outdir is None:
        outdir = dtstr
    os.makedirs(outdir, exist_ok=True)
    if buck_name is None:
        buck_name = os.getenv('UKMONLIVEBUCKET', default='s3://ukmon-live')[5:]
    s3 = boto3.client('s3')
    print(f'looking for {dtstr} in {buck_name}')
    try:
        x = s3.list_objects_v2(Bucket=buck_name,Prefix=f'M{dtstr}')
        if x['KeyCount'] > 0:
            print(f"found {x['KeyCount']} records, saving to {outdir}")
            for k in x['Contents']:
                key = k['Key']
                if '.xml' in key:
                    s3.download_file(buck_name, key, os.path.join(outdir, key))
                    x = ufoc.UCXml(os.path.join(outdir, key))
                    fn = x.ucxml['ufocapture_record']['@cap'].strip()
                    os.remove(os.path.join(outdir, key))
                    key = key.replace('.xml', 'P.jpg')
                    if len(fn) < 5:
                        outkey = key
                        #spls = key.split('_')
                        #stationid = spls[-1][:6].lower()
                        #dtime = key[1:16]
                        #patt = f'FF_{stationid}_{dtime}'
                    else:
                        outkey = fn.replace('.fits', '.jpg')
                        #patt = fn[:26]
                        #stationid = fn[3:9].lower()
                    print(key)
                    s3.download_file(buck_name, key, os.path.join(outdir, outkey))
                    if create_txt is True:
                        createTxtFile(key, outdir)
        else:
            print('no records found')
    except:
        print('Error accessing AWS S3 - do you have access?')


def getFBFiles(patt, outdir='.'):
    """
    Retrieve fireball files from the ukmon website that match a pattern

    Arguments:
        patt:      [str] pattern to match
        outdir:     [str] Where to save the file. Default is '.''

    Notes:
        This function will fail if you do not have access to the ukmon-shared bucket. 
    """
    buck_name = os.getenv('UKMONSHAREDBUCKET', default='s3://ukmon-shared')[5:]
    s3 = boto3.client('s3')
    print(f'looking for {patt} in {buck_name}')
    fullpatt = f'fireballs/interesting/{patt}'
    try:
        x = s3.list_objects_v2(Bucket=buck_name,Prefix=fullpatt)
        if x['KeyCount'] > 0:
            for k in x['Contents']:
                key = k['Key']
                _, fname = os.path.split(key)
                print(f"saving {fname} to {outdir.lower()}")
                s3.download_file(buck_name, key, os.path.join(outdir, fname))
        # platepar file
        _, camid = os.path.split(outdir)
        fullpatt = f'consolidated/platepars/{camid}'
        x = s3.list_objects_v2(Bucket=buck_name,Prefix=fullpatt)
        if x['KeyCount'] > 0:
            for k in x['Contents']:
                key = k['Key']
                fname = 'platepar_cmn2010.cal'
                print(f"saving {fname} to {outdir.lower()}")
                s3.download_file(buck_name, key, os.path.join(outdir, fname))
        # config file
        dtstr = patt[10:25]
        gotcfg = False
        fullpatt = f'matches/RMSCorrelate/{camid}/{camid}_{dtstr[:8]}'
        x = s3.list_objects_v2(Bucket=buck_name,Prefix=fullpatt)
        if x['KeyCount'] > 0:
            for k in x['Contents']:
                key = k['Key']
                fname = '.config'
                if fname in key:
                    print(f"saving {fname} to {outdir.lower()}")
                    s3.download_file(buck_name, key, os.path.join(outdir, fname))
                    gotcfg = True
        if gotcfg is False:
            dt = datetime.datetime.strptime(dtstr, '%Y%m%d_%H%M%S')
            dtstr = (dt +datetime.timedelta(days = -1)).strftime('%Y%m%d_%H%M%S')
            fullpatt = f'matches/RMSCorrelate/{camid}/{camid}_{dtstr[:8]}'
            x = s3.list_objects_v2(Bucket=buck_name,Prefix=fullpatt)
            if x['KeyCount'] > 0:
                for k in x['Contents']:
                    key = k['Key']
                    fname = '.config'
                    if fname in key:
                        print(f"saving {fname} to {outdir.lower()}")
                        s3.download_file(buck_name, key, os.path.join(outdir, fname))
                        gotcfg = True
    except:
        print('Error accessing AWS S3 - do you have access?')
    return 


def createTxtFile(fname, outdir='.'):
    """
    Create a text file named after the cameraID, containing a list of fireball files 
    to be retrieved from a remote camera

    Arguments:
        fname:  [str] the name of the FF file to be retrieved
        outdir: [str] where to save the files. Default '.'

    Notes:
        the fname parameter should be the name of the live JPG for which you wish to 
        retrieve the corresponding FF and FR files.
    """
    if fname[0] == 'M':
        spls = fname.split('_')
        stationid = spls[-1][:6].lower()
        dtime = fname[1:16]
        patt = f'FF_{stationid}_{dtime}'
        stationid = stationid.lower()
    else:
        patt = fname[:25]
        stationid = fname[3:9].lower()
    txtf = os.path.join(outdir, f'{stationid}.txt')
    if os.path.isfile(txtf):
        os.remove(txtf)
    patt = patt.upper()
    with open(txtf,'w') as outf:
        outf.write(f'{patt}\n{patt.replace("FF_", "FR_")}\n')
    return txtf


if __name__ == '__main__':
    getLiveJpgs(sys.argv[1])



# Copyright (C) 2018-2023 Mark McIntyre
# flake8: noqa

try: 
    from .multiDayRadiant import multiDayRadiant
except Exception:
    print('MultiDayRadiant can only be run from the RMS source folder')

try: 
    from .multiTrackStack import multiTrackStack
except Exception:
    print('multiTrackStack can only be run from an RMS environment')
try: 
    from .analyseUFOwithRMS import analyseUFOwithRMS
except Exception:
    print('analyseUFOwithRMS can only be run from an RMS environment')

try:
    from .multiEventGroundMap import multiEventGroundMap
except Exception:
    print('multiEventGroundMap can only be run in a WMPL environment')
try:
    from .plotCAMSOrbits import plotCAMSOrbits
except Exception:
    print('plotCAMSOrbits can only be run in a WMPL environment')
try:
    from .plotRMSOrbits import plotRMSOrbits
except Exception:
    print('plotRMSOrbits can only be run in a WMPL environment')




# Copyright (C) 2018-2023 Mark McIntyre
# flake8: noqa
"""
Utility functions used across the UKMON toolset

List of Functions:
jd2Date, date2JD, datetime2JD, jd2DynamicalTimeJD, jd2LST, sollon2jd
greatCircleDistance, angleBetweenSphericalCoords, calcApparentSiderealEarthRotation
calcNutationComponents, equatorialCoordPrecession,  raDec2AltAz, altAz2RADec
altAz2RADec_vect, raDec2AltAz_vect, equatorialCoordPrecession_vect
annotateImage, annotateImageArbitrary
getActiveShowers, getActiveShowersStr, getShowerDets, getShowerPeak
trackCsvtoKML, getTrackDetails, getTrajPickle, munchKML
sendAnEmail
shortestDistance2Lines
"""

from .Math import jd2Date, date2JD,datetime2JD, jd2DynamicalTimeJD, JULIAN_EPOCH, J2000_JD, jd2LST
from .Math import greatCircleDistance, angleBetweenSphericalCoords, calcApparentSiderealEarthRotation
from .Math import calcNutationComponents, equatorialCoordPrecession,  raDec2AltAz, altAz2RADec
from .Math import altAz2RADec_vect, raDec2AltAz_vect, equatorialCoordPrecession_vect

from .annotateImage import annotateImage, annotateImageArbitrary

from .convertSolLon import sollon2jd

from .getActiveShowers import getActiveShowers, getActiveShowersStr

from .getShowerDates import getShowerDets, getShowerPeak
#from .getShowerDates import loadJenniskensShowers, loadDataFile, loadFullData, refreshShowerData
#from .getShowerDates import loadLookupTable

from .kmlHandlers import trackCsvtoKML, getTrackDetails, getTrajPickle, munchKML

from .sendAnEmail import sendAnEmail



# Copyright (C) 2018-2023 Mark McIntyre
# flake8: noqa
"""
Functions to load and manage various meteor file formats

loadFTPDetectInfo
WriteNewFTPFile
filterFTPforSpecificTime
MeteorObservation

IMOShowerList
majorlist
minorlist

loadPlatepars
platepar

UAXml
UCXml

"""

from .ftpDetectInfo import filterFTPforSpecificTime, writeNewFTPFile, loadFTPDetectInfo, MeteorObservation
from .imoWorkingShowerList import IMOshowerList, majorlist, minorlist
from .platepar import loadPlatepars, platepar
from .ReadUFOAnalyzerXML import UAXml
from .ReadUFOCapXML import UCXml



# Copyright (C) 2018-2023 Mark McIntyre
# flake8: noqa

from .apiExampleCode import apiurl, matchApiCall, detailApiCall1, detailApiCall2
from .drawFTPfile import drawFTPFile
from .findNearDuplicates import findNearDuplicates
from .getLiveImages import getLiveJpgs, getFBFiles, createTxtFile
from .getOverlappingFovs import checkKMLOverlap, pointInsideFov, getOverlapWith, getOverlappingCameras
from .plotTrack import trackToDistvsHeight, trackToTimevsVelocity, trackToTimevsHeight
from .retrieveECSV import getECSVs


#
# Example showing how to use the matchdata API from Python
#
# Copyright (C) 2018-2023 Mark McIntyre

import pandas as pd

apiurl = 'https://api.ukmeteornetwork.co.uk/matches'


def matchApiCall(reqtyp, reqval):
    # get names of all matches for a given date
    apicall = '{}?reqtyp={}&reqval={}'.format(apiurl, reqtyp, reqval)
    matchlist = pd.read_json(apicall, lines=True)
    return matchlist


def detailApiCall1(reqtyp, reqval):
    # get details of one event
    apicall = '{}?reqtyp={}&reqval={}'.format(apiurl, reqtyp, reqval)
    evtdetail = pd.read_json(apicall, typ='series')
    return evtdetail


def detailApiCall2(reqtyp, matchlist):
    # get details for the first five events in the match list
    # and put them in a pandas dataframe, then sort by brightest
    details = []
    for id in matchlist.head(5).orbname:
        reqval = id
        apicall = '{}?reqtyp={}&reqval={}'.format(apiurl, reqtyp, reqval)
        details.append(pd.read_json(apicall, typ='series'))
    df = pd.DataFrame(details)
    df = df.sort_values(by=['_mag'])
    return df


if __name__ == '__main__':
    # get all matched events for a given date
    reqtyp = 'matches'
    reqval = '20211121'
    matchlist = matchApiCall(reqtyp, reqval)
    print(matchlist)

    reqtyp = 'detail'
    reqval = '20211121_032219.699_UK'
    evtlist = detailApiCall1(reqtyp, reqval)
    print(evtlist)

    reqtyp = 'detail'
    evt = detailApiCall2(reqtyp, matchlist)
    print(evt)



import xmltodict
from shapely.geometry import Polygon
import csv
import simplekml
import numpy as np
import pandas as pd
import os
import sys
import boto3

try:
    from wmpl.Utils.Pickling import loadPickle
    gotpickler = True
except:
    gotpickler = False


def munchKML(kmlFilename, return_poly=False):
    with open(kmlFilename) as fd:
        x = xmltodict.parse(fd.read())
        cname = x['kml']['Folder']['name']
        coords = x['kml']['Folder']['Placemark']['MultiGeometry']['Polygon']['outerBoundaryIs']['LinearRing']['coordinates']
        coords = coords.split('\n')
        if return_poly is False:
            lats = []
            lngs = []
            for lin in coords:
                s = lin.split(',')
                lngs.append(float(s[0]))
                lats.append(float(s[1]))
            return cname, lats, lngs
        else:
            ptsarr=[]
            for lin in coords:
                s = lin.split(',')
                ptsarr.append((float(s[0]), float(s[1])))
            polyg = Polygon(ptsarr)
            return cname, polyg 


def trackCsvtoKML(trackcsvfile, trackdata=None, saveOutput=True):
    """ 
    Either reads a CSV file containing lat, long, height, time of an event and 
    creates a 3d KML file from it or, if trackdata is populated, converts a Pandas dataframe containing
    the same data. Output is written to disk unless saveOutput is false. 
    
    Arguments:
        trackcsvfile:   [string] full path to the file to read from
        trackdata:      [array] pandas dataframe containing the data. Default None
        saveOutput:     [bool] write the KML file to disk. Default true

    Returns:
        the KML file as a tuple
        """
    kml=simplekml.Kml()
    kml.document.name = trackcsvfile
    if trackdata is None:
        inputfile = csv.reader(open(trackcsvfile))
        for row in inputfile:
            #columns are lat, long, height, times
            kml.newpoint(name='', coords=[(row[1], row[0], row[2])])
    else:
        for i,r in trackdata.iterrows():
            kml.newpoint(name='', coords=[(r[1], r[0], r[2])], extrude=1, altitudemode='absolute')
    outname = trackcsvfile.replace('.csv','.kml')
    if saveOutput:
        kml.save(outname)
    return kml


def getTrackDetails(traj):
    lats = []
    lons = []
    alts = [] 
    lens = []
    # Go through observation from all stations
    for obs in traj.observations:
        # Go through all observed points
        for i in range(obs.kmeas):
            lats.append(np.degrees(obs.model_lat[i]))
            lons.append(np.degrees(obs.model_lon[i]))
            alts.append(obs.model_ht[i])
            lens.append(obs.time_data[i])
    df = pd.DataFrame({"lats": lats, "lons": lons, "alts": alts, "times": lens})
    df = df.sort_values(by=['times', 'lats'])
    return df


def getTrajPickle(trajname):
    tmpdir = os.getenv('TMP', default='/tmp')
    s3 = boto3.client('s3')
    srcbucket = 'ukmon-shared'
    orb = trajname
    picklefile = orb[:15] + '_trajectory.pickle'
    kmlfile = orb[:15] + '.kml'
    yr = picklefile[:4]
    ym = picklefile[:6]
    ymd = picklefile[:8]
    fname = f'matches/RMSCorrelate/trajectories/{yr}/{ym}/{ymd}/{orb}/{picklefile}'
    localfile = os.path.join(tmpdir, picklefile)
    s3.download_file(srcbucket, fname, localfile)
    traj = loadPickle(*os.path.split(localfile))
    try:
        os.remove(localfile)
    except:
        print(f'unable to remove {localfile}')
    return traj, kmlfile


if __name__ == '__main__':
    ext = os.path.splitext(sys.argv[1])
    if ext == 'csv':
        trackCsvtoKML(sys.argv[1])
    else:
        if gotpickler is False:
            print('WMPL not available, aborting')
        else:
            traj, kmlfile = getTrajPickle(sys.argv[1])
            tdets = getTrackDetails(traj)
            trackCsvtoKML(kmlfile, tdets)
